#include "DeerStudio/EditorEngine/API/EditorEngine_Functions.h"
#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/EditorEngine_ErrorHandle.h"

#include "angelscript.h"

namespace Deer {
    void EditorEngine::registerEditorEngineFunctions() {
        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool menuItem(const string& in)",
            asFUNCTION(menuItem),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void textColor(float, float, float, const string& in)",
            asFUNCTION(textColor),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void text(const string& in)",
            asFUNCTION(text),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void drawIcon(const string& in, int)",
            asFUNCTION(drawIcon),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool isMouseClicked(int)",
            asFUNCTION(
                isMouseClicked
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool isMouseDoubleClicked(int)",
            asFUNCTION(
                isMouseDoubleClicked
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void setupAutomaticColumns(int)",
            asFUNCTION(
                setupAutomaticColumns
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void endColumns()",
            asFUNCTION(endColumns),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void nextColumn()",
            asFUNCTION(nextColumn),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void print(const string& in)",
            asFUNCTION (print),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void textCentered(const string& in)",
            asFUNCTION(textCentered),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void drawIconCentered(const string& in, int)",
            asFUNCTION(drawIconCentered),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "int getResourceCount(ResourceType, const string& in)",
            asFUNCTION(getResourceCount),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getResourceNameById(ResourceType, const string& in, int)",
            asFUNCTION(getResourceNameById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getResourcePathById(ResourceType, const string& in, int)",
            asFUNCTION(getResourcePathById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "int getDirCount(ResourceType, const string& in)",
            asFUNCTION(getDirCount),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getDirPathById(ResourceType, const string& in, int)",
            asFUNCTION(getDirPathById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getDirNameById(ResourceType, const string& in, int)",
            asFUNCTION(getDirNameById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void treeNode(const string& in, int)",
            asFUNCTION(treeNode),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterFuncdef("void CallbackFunc(any@)"));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void treeNode(const string& in, int, any@+, CallbackFunc@+)",
            asFUNCTION(treeNodeRecursive),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "Entity getRoot()",
            asFUNCTION(getRoot),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterObjectMethod(
            "Entity",
            "int getChildCount()",
            asMETHOD(EntityStruct, getChildCount),
            asCALL_THISCALL
        ));

        AS_CHECK(scriptEngine->RegisterObjectMethod(
            "Entity",
            "Entity getChild(int)",
            asMETHOD(EntityStruct, getChild),
            asCALL_THISCALL
        ));

        AS_CHECK(scriptEngine->RegisterObjectMethod(
            "Entity",
            "string getName()",
            asMETHOD(EntityStruct, getName),
            asCALL_THISCALL
        ));
        
        AS_CHECK(scriptEngine->RegisterObjectMethod(
            "Entity",
            "int getId()",
            asMETHOD(EntityStruct, getId),
            asCALL_THISCALL
        ));
    }
}